-- backend of the track sorter.  responsible for pulling tracks out of
-- two small (distributed) dual port rams.  LOWEST specific design.

-- jamieson olsen 3 jan 2002

-- 1. reset the pointer to 0.  
-- 2. pull out the track in ram A.  
-- 3. if the MSb is clear, goto 5.
-- 4. pull track out of ram B.
-- 5. increment pointer.
-- 6. goto 2.
-- revised 15 Feb 2005: bug fix S.X.Wu

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity low_dpr_read is
 	port (
		clock:		in	std_logic;						-- 53MHz
		clear:		in	std_logic;						-- active high
		data_a:		in 	std_logic_vector(11 downto 0);	-- dout from RAMs
		data_b:		in	std_logic_vector(11 downto 0);

		ptr:		out	std_logic_vector(3  downto 0);	-- address pointer to RAMs
		ctoc_trk:	out	std_logic_vector(10 downto 0)	-- track out VCEeePppppp
	);
end low_dpr_read;


architecture low_dpr_read_arch of low_dpr_read is

	-- define registers...

		signal ptr_reg: 		std_logic_vector( 3 downto 0);
		signal trk_out_reg:		std_logic_vector(11 downto 0);	-- SVCEeePppppp
		signal incr_reg: 		std_logic;
	
	-- define signals...

		signal incr:			std_logic;		-- increment the pointer
		signal trk_mux:			std_logic_vector(11 downto 0);

	-- declare components...

begin

	-- combinatorial stuff...

		incr <= '0' when (data_a(11)='1' and incr_reg='1') else '1';

-- modified 2-15-2005 this ensures that after rpt_reg change, data_a	always
-- get read out first
--		trk_mux(11 downto 0) <= data_b(11 downto 0) when (trk_out_reg(11)='1') else
		trk_mux(11 downto 0) <= data_b(11 downto 0) when (incr_reg = '0') else
								data_a(11 downto 0);

		ctoc_trk(10 downto 0) 	<= trk_out_reg(10 downto 0);

		ptr(3 downto 0)		 	<= ptr_reg(3 downto 0);

	-- sequential stuff...

		pointer_control: process(clock)
		 begin
			if rising_edge(clock) then

-- bug fix 2-15-2005 S.X.Wu
-- 				incr_reg <= incr;
 				incr_reg <= incr or clear;
				trk_out_reg(11 downto 0) <= trk_mux(11 downto 0);

				if (clear='1') then
					ptr_reg(3 downto 0) <= "0000";
				elsif (incr='1') then
					ptr_reg(3 downto 0) <= ptr_reg(3 downto 0) + 1;
				else
					ptr_reg(3 downto 0) <= ptr_reg(3 downto 0);
				end if;

			end if;
		end process;


	-- instantiate components...

end low_dpr_read_arch;
