-- gate_mux704.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***
--  28 Jun 1999:    created using Virtex TBUFs
--   9 Jan 2002:    modifed the order so that sel(0) will select d(21 downto 0) now.
--  27 Feb 2002:    Synplify cannot synthsize the tri-states correctly (!) It keeps wanting to
--                  build the MUX out of LUTs!  Rewrite until it gets it right.
--  28 Feb 2002:    Re-write using conventional gates.  This may get ugly!
--  31 Mar 2004:    XST having trouble with too many logic levels.  Revert back to using TBUFS.
--  01 Jul 2004:    changed to synchronous reset
--                  tri-state control signal simplified S.X.Wu
library ieee;
use ieee.std_logic_1164.all;

entity mux704 is
    port (
        d:      in  std_logic_vector(703 downto 0);     -- matrix flops
        sel:    in  std_logic_vector(31 downto 0);      -- one hot select lines
        reset:  in  std_logic;                          -- active low
        clock:  in  std_logic;                          -- 53MHz
        q:      out std_logic_vector(21 downto 0)
    );
end mux704;

architecture mux704_arch of mux704 is

    -- define registers...

        signal mux_out_reg: std_logic_vector(21 downto 0);

    -- define signals...

        signal mux_out:     std_logic_vector(21 downto 0);

 begin

    -- combinatorial stuff...

        q(21 downto 0)       <= mux_out_reg(21 downto 0);
-- since sel is one-hot it could be simplified 1-jul-2004 S.X.Wu
i_gen: for i in sel'range generate
      mux_out <= d(i*22+21 downto i*22) when sel(i) = '1' else (others => 'Z');
end generate; 

--		mux_out(21 downto 0) <= d( 21 downto   0) when (sel(31 downto 0)="00000000000000000000000000000001") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d( 43 downto  22) when (sel(31 downto 0)="00000000000000000000000000000010") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d( 65 downto  44) when (sel(31 downto 0)="00000000000000000000000000000100") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d( 87 downto  66) when (sel(31 downto 0)="00000000000000000000000000001000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(109 downto  88) when (sel(31 downto 0)="00000000000000000000000000010000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(131 downto 110) when (sel(31 downto 0)="00000000000000000000000000100000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(153 downto 132) when (sel(31 downto 0)="00000000000000000000000001000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(175 downto 154) when (sel(31 downto 0)="00000000000000000000000010000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(197 downto 176) when (sel(31 downto 0)="00000000000000000000000100000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(219 downto 198) when (sel(31 downto 0)="00000000000000000000001000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(241 downto 220) when (sel(31 downto 0)="00000000000000000000010000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(263 downto 242) when (sel(31 downto 0)="00000000000000000000100000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(285 downto 264) when (sel(31 downto 0)="00000000000000000001000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(307 downto 286) when (sel(31 downto 0)="00000000000000000010000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(329 downto 308) when (sel(31 downto 0)="00000000000000000100000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(351 downto 330) when (sel(31 downto 0)="00000000000000001000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(373 downto 352) when (sel(31 downto 0)="00000000000000010000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(395 downto 374) when (sel(31 downto 0)="00000000000000100000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(417 downto 396) when (sel(31 downto 0)="00000000000001000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(439 downto 418) when (sel(31 downto 0)="00000000000010000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(461 downto 440) when (sel(31 downto 0)="00000000000100000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(483 downto 462) when (sel(31 downto 0)="00000000001000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(505 downto 484) when (sel(31 downto 0)="00000000010000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(527 downto 506) when (sel(31 downto 0)="00000000100000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(549 downto 528) when (sel(31 downto 0)="00000001000000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(571 downto 550) when (sel(31 downto 0)="00000010000000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(593 downto 572) when (sel(31 downto 0)="00000100000000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(615 downto 594) when (sel(31 downto 0)="00001000000000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(637 downto 616) when (sel(31 downto 0)="00010000000000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(659 downto 638) when (sel(31 downto 0)="00100000000000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(681 downto 660) when (sel(31 downto 0)="01000000000000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";
--		mux_out(21 downto 0) <= d(703 downto 682) when (sel(31 downto 0)="10000000000000000000000000000000") else "ZZZZZZZZZZZZZZZZZZZZZZ";

    -- sequential stuff...
-- change to sychronous reset 1-jul-2004 S.X.Wu
process(clock)
begin
  if(clock'event and clock = '1')then
    if(reset = '0')then
	   mux_out_reg <= (others => '0');
	 else
	   mux_out_reg <= mux_out;
	 end if;
  end if;
end process;
--        process(clock, reset)
--         begin
--            if (reset='0') then
--                mux_out_reg(21 downto 0) <= "0000000000000000000000";
--            elsif rising_edge(clock) then
--mux_out_reg(21 downto 0) <= mux_out(21 downto 0);
--   end if;
--end process;

end mux704_arch;
