-- comp_b.vhd
-- jamieson olsen <jamieson@fnal.gov>
-- 28 Jan 2002

-- revised 31 Jan 2002 : make the width field 3 bits, not 2.
-- revised 14 Aug 2002 : expand width of bus by 1.
-- revised 20 Oct 2002 : add reset

-- bit 5 of each input is a selection bit; don't modify just pass it
-- bit 4 of each input is a valid bit
-- bits 3..0 width field

-- A(5..0)  	B(5..0)		action
-- -------		-------		------
--  A1yyyy		B1zzzz		if (yyyy<=zzzz) then pass 0A1yyyy else 1B1zzzz
--  A1yyyy		B0----		pass 0A1yyyy
--  A0----		B1zzzz		pass 1B1zzzz
--  -0----		-0----	    pass 000000

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity comp_b is
	port(
		clock:		in	std_logic;
		reset:		in	std_logic;
		a: 			in 	std_logic_vector(5 downto 0);	-- sel + valid + n
		b:			in	std_logic_vector(5 downto 0);	-- sel + valid + n
		q:			out	std_logic_vector(6 downto 0)	-- sel(1..0) + valid + |difference|
	);
end comp_b;

architecture comp_b_arch of comp_b is

	-- define registers...

		signal q_reg: 	std_logic_vector(6 downto 0);

	-- define signals

		signal smaller_a:	std_logic;
		signal sel_a:		std_logic;
		signal sel_b:		std_logic;
		signal mux_out:		std_logic_vector(6 downto 0);
	
 begin

	-- combinatorial stuff...

		-- compare values...

		smaller_a	<=	'1' when (a(3 downto 0) < b(3 downto 0)) else
						'1' when (a(3 downto 0) = b(3 downto 0)) else
						'0';

		sel_a		<=	'1' when (a(4)='1' and b(4)='1' and smaller_a='1') else
						'1' when (a(4)='1' and b(4)='0'                  ) else
						'0';

		sel_b		<=	'1' when (a(4)='1' and b(4)='1' and smaller_a='0') else
						'1' when (a(4)='0' and b(4)='1'                  ) else
						'0';

		-- mux operation (sel_a, sel_b)

			-- 00 = pass 000000
			-- 10 = pass 0+A
			-- 01 = pass 1+B
			-- 11 = can never happen, but pass 00000 anyway...

		mux_out(6 downto 0)	<= 	('0'&a(5 downto 0)) when (sel_a='1' and sel_b='0') else
								('1'&b(5 downto 0)) when (sel_a='0' and sel_b='1') else
								"0000000";

		q(6 downto 0)		<= q_reg(6 downto 0);


	-- sequential stuff...

--		process(clock, reset) 	 	change to sync reset Aug-06-04
		process(clock) 
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				q_reg(6 downto 0)	<= "0000000";
			 else
				q_reg(6 downto 0)	<= mux_out(6 downto 0);
			 end if;
		  end if;
		end process;

end comp_b_arch;




