-- pt_adder.vhd
-- jamieson olsen <jamieson@fnal.gov>

--	reference my engineering note 2002-02-08B!

-- *** revision history *** 
--
-- 08 Feb 2002:	  	created and functionally simulated.  NOTE: using the DFEA internal
--					Pt and Extended Pt encoding...
-- 11 Feb 2002:		expand to 8-bit adder
-- 14 Aug 2002: 	change pt bin identifiers: 00=max, 11=low, add different values for low.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity pt_adder is
	port(
		reset:		in 	std_logic;						-- active low
		clock:		in 	std_logic;		
		pt:			in	std_logic_vector(1 downto 0);	-- 00=max, 11=low
		ext_pt:		in	std_logic_vector(2 downto 0);	-- extended Pt field
		valid:		in	std_logic;						-- active high
		start:		in	std_logic;						-- active high

		sum_pt:		out	std_logic_vector(7 downto 0)
	);
end pt_adder;

architecture pt_adder_arch of pt_adder is

	-- define signals...

		signal gev_lut:			std_logic_vector(7 downto 0);
	
	-- define registers...

		signal gev_reg:		std_logic_vector(7 downto 0);
		signal sum_reg:		std_logic_vector(7 downto 0);
		signal start_reg:	std_logic;
	
 begin

	-- combinatorial stuff...

		gev_lut(7 downto 0)	<=	"01010000" when (pt(1 downto 0)="00" and ext_pt(2 downto 0)="000") else		-- max 
								"00011011" when (pt(1 downto 0)="00" and ext_pt(2 downto 0)="001") else
								"00010000" when (pt(1 downto 0)="00" and ext_pt(2 downto 0)="010") else
								"00001100" when (pt(1 downto 0)="00" and ext_pt(2 downto 0)="011") else

								"00001001" when (pt(1 downto 0)="01" and ext_pt(2 downto 0)="000") else		-- high
								"00000111" when (pt(1 downto 0)="01" and ext_pt(2 downto 0)="001") else
								"00000110" when (pt(1 downto 0)="01" and ext_pt(2 downto 0)="010") else
								"00000101" when (pt(1 downto 0)="01" and ext_pt(2 downto 0)="011") else

								"00000100" when (pt(1 downto 0)="10" and ext_pt(2 downto 0)="000") else		-- med
								"00000100" when (pt(1 downto 0)="10" and ext_pt(2 downto 0)="001") else
								"00000011" when (pt(1 downto 0)="10" and ext_pt(2 downto 0)="010") else
								"00000011" when (pt(1 downto 0)="10" and ext_pt(2 downto 0)="011") else

								"00000011" when (pt(1 downto 0)="11" and ext_pt(2 downto 0)="000") else		-- low
								"00000011" when (pt(1 downto 0)="11" and ext_pt(2 downto 0)="001") else
								"00000010" when (pt(1 downto 0)="11" and ext_pt(2 downto 0)="010") else
								"00000010" when (pt(1 downto 0)="11" and ext_pt(2 downto 0)="011") else
								"00000001" when (pt(1 downto 0)="11" and ext_pt(2 downto 0)="100") else
								"00000000";																


		sum_pt(7 downto 0)	<=	sum_reg(7 downto 0);

	-- sequential stuff...

--	process(clock, reset)	change to sync reset Aug-06-04
	process(clock)
 	 begin

		if rising_edge(clock) then
		 if (reset='0') then 

			gev_reg(7 downto 0)		<= 	"00000000";
			start_reg				<=	'0';
			sum_reg(7 downto 0)		<= 	"00000000";
		 else
			start_reg	<= 	start;

			if (valid='1') then
				gev_reg(7 downto 0)	<= 	gev_lut(7 downto 0);
			else
				gev_reg(7 downto 0)	<= 	"00000000";
			end if;

			if (start_reg='1') then		-- load
				sum_reg(7 downto 0)	<=	gev_reg(7 downto 0);	
			else						-- add
				sum_reg(7 downto 0)	<=	gev_reg(7 downto 0) + sum_reg(7 downto 0);
			end if;

		 end if;
	  end if;
	end process;

end pt_adder_arch;
