-- 11 x 16 dual port ram for Xilinx FPGAs.
-- distributed RAM, so this is pseudo-dual port operation!
-- 16 words of 11-bits each.

-- PORT A is SYNC write only.  DIN written on rising edge of WCLK if WE=1
-- PORT B is ASYNC read only.  DOUT valid a few ns after ADDRB is stable.

-- jamieson olsen 14 Jan 2002


library ieee;
use ieee.std_logic_1164.all;

--library virtex;

entity ram11x16d is
 	port (
		wclk:		in	std_logic;						-- sync write clk
		we:			in	std_logic;						-- active high
		addra:		in	std_logic_vector( 3 downto 0);	-- write port address
		din:		in	std_logic_vector(10 downto 0);

		addrb:		in	std_logic_vector( 3 downto 0);	-- read port address
		dout:		out	std_logic_vector(10 downto 0)	-- read data
	);
end ram11x16d;


architecture ram11x16d_arch of ram11x16d is

	-- define registers...

	-- define signals...

	-- declare components...

	component RAM16X1D
		port(
			we:		in	std_logic;
			d:		in	std_logic;
			wclk:	in	std_logic;
			a0:		in	std_logic;
			a1:		in	std_logic;
			a2:		in	std_logic;
			a3:		in	std_logic;
			dpra0:	in	std_logic;
			dpra1:	in	std_logic;
			dpra2:	in	std_logic;
			dpra3:	in	std_logic;
			spo:	out	std_logic;
			dpo:	out	std_logic
		);
	end component;

attribute box_type : string;
attribute box_type of RAM16X1D : component is "black_box";

begin

	-- combinatorial stuff...

	-- sequential stuff...

	-- instantiate components...

	bit0: RAM16X1D
		port map(
			we		=> we,
			d		=> din(0),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(0)
		);

	bit1: RAM16X1D
		port map(
			we		=> we,
			d		=> din(1),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(1)
		);

	bit2: RAM16X1D
		port map(
			we		=> we,
			d		=> din(2),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(2)
		);

	bit3: RAM16X1D
		port map(
			we		=> we,
			d		=> din(3),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(3)
		);

	bit4: RAM16X1D
		port map(
			we		=> we,
			d		=> din(4),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(4)
		);

	bit5: RAM16X1D
		port map(
			we		=> we,
			d		=> din(5),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(5)
		);

	bit6: RAM16X1D
		port map(
			we		=> we,
			d		=> din(6),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(6)
		);

	bit7: RAM16X1D
		port map(
			we		=> we,
			d		=> din(7),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(7)
		);

	bit8: RAM16X1D
		port map(
			we		=> we,
			d		=> din(8),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(8)
		);

	bit9: RAM16X1D
		port map(
			we		=> we,
			d		=> din(9),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(9)
		);

	bit10: RAM16X1D
		port map(
			we		=> we,
			d		=> din(10),
			wclk	=> wclk,
			a0		=> addra(0),
			a1		=> addra(1),
			a2		=> addra(2),
			a3		=> addra(3),
			dpra0	=> addrb(0),
			dpra1	=> addrb(1),
			dpra2	=> addrb(2),
			dpra3	=> addrb(3),
			spo		=> open,
			dpo		=> dout(10)
		);

end ram11x16d_arch;
