-- cluster_top.vhd
-- CPS axial cluster finder
-- jamieson olsen <jamieson@fnal.gov>

-- see eng. notes 2002-01-22a and 2002-01-30a for details.

-- *** revision history ***

-- 20 march 2002: created, functionally simulated.

library ieee;
use ieee.std_logic_1164.all;

entity cluster_top is
 	port(
	    cps:		in  std_logic_vector(31 downto 0);
		clock:		in	std_logic;
	    load:       in  std_logic;
	    reset:      in  std_logic;
	
		valid_a:	out	std_logic;
		width_a:	out	std_logic_vector(3 downto 0);
		center_a:	out std_logic_vector(3 downto 0);
	
		valid_b:	out	std_logic;
		width_b:	out	std_logic_vector(3 downto 0);
		center_b:	out std_logic_vector(3 downto 0)
    );
end cluster_top;


architecture cluster_top_arch of cluster_top is

	-- define registers...

	-- define signals...

		signal a_inner:			std_logic_vector(15 downto 0);
		signal a_outer:			std_logic_vector(15 downto 0);
		signal b_inner:			std_logic_vector(15 downto 0);
		signal b_outer:			std_logic_vector(15 downto 0);

	-- declare components...

		component cps_edge
		 	port (
			    cps:    in  std_logic_vector(31 downto 0);
			    a_in:  	out std_logic_vector(15 downto 0);
				a_out:	out std_logic_vector(15 downto 0);
			    b_in:  	out std_logic_vector(15 downto 0);
				b_out:	out std_logic_vector(15 downto 0)
		    );
		end component;

		component cluster_finder_a
			port(
			    inner_edge:	in  std_logic_vector(15 downto 0);
			    outer_edge:	in  std_logic_vector(15 downto 0);
				clock:		in	std_logic;
			    load:       in  std_logic;
			    reset:      in  std_logic;			
				valid:		out	std_logic;
				width:		out	std_logic_vector(3 downto 0);
				center:		out std_logic_vector(3 downto 0)
		    );
		end component;

		component cluster_finder_b
			port(
			    inner_edge:	in  std_logic_vector(15 downto 0);
			    outer_edge:	in  std_logic_vector(15 downto 0);
				clock:		in	std_logic;
			    load:       in  std_logic;
			    reset:      in  std_logic;			
				valid:		out	std_logic;
				width:		out	std_logic_vector(3 downto 0);
				center:		out std_logic_vector(3 downto 0)
		    );
		end component;

 begin

	-- combinatorial stuff...

    -- sequential stuff...

	-- instantiate components...

		cps_edge_detect: cps_edge
		 	port map(
			    cps(31 downto 0)	=> cps(31 downto 0),	
			    a_in(15 downto 0)	=> a_inner(15 downto 0),
				a_out(15 downto 0)	=> a_outer(15 downto 0),
			    b_in(15 downto 0)	=> b_inner(15 downto 0),
				b_out(15 downto 0)	=> b_outer(15 downto 0)
		    );

		finder_a: cluster_finder_a
			port map(
			    inner_edge(15 downto 0)	=> a_inner(15 downto 0),
			    outer_edge(15 downto 0)	=> a_outer(15 downto 0), 
				clock					=> clock,
			    load					=> load,
			    reset					=> reset,
				valid					=> valid_a,
				width(3 downto 0)		=> width_a(3 downto 0),
				center(3 downto 0)		=> center_a(3 downto 0)
		    );

		finder_b: cluster_finder_b
			port map(
			    inner_edge(15 downto 0)	=> b_inner(15 downto 0),
			    outer_edge(15 downto 0)	=> b_outer(15 downto 0), 
				clock					=> clock,
			    load					=> load,
			    reset					=> reset,
				valid					=> valid_b,
				width(3 downto 0)		=> width_b(3 downto 0),
				center(3 downto 0)		=> center_b(3 downto 0)
		    );



end cluster_top_arch;
