-- low pt track equations
-- doublets in, matrix out.  fully combinatorial, no registers.
-- optimize for small area when compiling

-- jamieson olsen <jamieson@fnal.gov> 
-- Automated version for as-built geometry 
--   Graham W. Wilson <graham@fnal.gov>  March 12th 2002
--            eqn IDs added     March 4th 2004
-- Input file  : low_tsector05.sorted                    
-- Output file : low_tsector05.vhdl                      

-- low design, full sector

library ieee;
  use ieee.std_logic_1164.all;

entity highpt_tracks is
    port(
		link0_array : IN std_logic_vector(195 downto 0);
		link2_array : IN std_logic_vector(195 downto 0);
		link3_array : IN std_logic_vector(195 downto 0);
		link4_array : IN std_logic_vector(195 downto 0);
		link5_array : IN std_logic_vector(195 downto 0);
		link6_array : IN std_logic_vector(195 downto 0);          
      eqs_vers: out std_logic_vector(31 downto 0);
      m: out std_logic_vector(351 downto 0)
    );
end highpt_tracks;

architecture track_arch of highpt_tracks is
component singlets
port(
     link0 : in std_logic_vector(195 downto 0);
     link2 : in std_logic_vector(195 downto 0);
     link3 : in std_logic_vector(195 downto 0);
     link4 : in std_logic_vector(195 downto 0);
     link5 : in std_logic_vector(195 downto 0);
     link6 : in std_logic_vector(195 downto 0);

     ai : out std_logic_vector(45 downto 2);
     ao : out std_logic_vector(46 downto 3);
     bi : out std_logic_vector(55 downto 4);
     bo : out std_logic_vector(56 downto 5);
     ci : out std_logic_vector(63 downto 8);
     co : out std_logic_vector(64 downto 9);     
     di : out std_logic_vector(70 downto 13);
     do : out std_logic_vector(71 downto 14);
     ei : out std_logic_vector(75 downto 20);
     eo : out std_logic_vector(76 downto 21);
     fi : out std_logic_vector(80 downto 27);
     fo : out std_logic_vector(81 downto 28);
     gi : out std_logic_vector(82 downto 37);
     go : out std_logic_vector(83 downto 38);     
     hi : out std_logic_vector(88 downto 43);
     ho : out std_logic_vector(89 downto 44)     
);
end component;
component high_tracks
    port(
     ai : in std_logic_vector(45 downto 2);
     ao : in std_logic_vector(46 downto 3);
     bi : in std_logic_vector(55 downto 4);
     bo : in std_logic_vector(56 downto 5);
     ci : in std_logic_vector(63 downto 8);
     co : in std_logic_vector(64 downto 9);     
     di : in std_logic_vector(70 downto 13);
     do : in std_logic_vector(71 downto 14);
     ei : in std_logic_vector(75 downto 20);
     eo : in std_logic_vector(76 downto 21);
     fi : in std_logic_vector(80 downto 27);
     fo : in std_logic_vector(81 downto 28);
     gi : in std_logic_vector(82 downto 37);
     go : in std_logic_vector(83 downto 38);     
     hi : in std_logic_vector(88 downto 43);
     ho : in std_logic_vector(89 downto 44);     
	  version : out std_logic_vector(15 downto 0);     
	  sector : out std_logic_vector(15 downto 0);     
        m: out std_logic_vector(351 downto 0)
    );
end component;
signal     ai : std_logic_vector(45 downto 2);
signal     ao : std_logic_vector(46 downto 3);
signal     bi : std_logic_vector(55 downto 4);
signal     bo : std_logic_vector(56 downto 5);
signal     ci : std_logic_vector(63 downto 8);
signal     co : std_logic_vector(64 downto 9);     
signal     di : std_logic_vector(70 downto 13);
signal     do : std_logic_vector(71 downto 14);
signal     ei : std_logic_vector(75 downto 20);
signal     eo : std_logic_vector(76 downto 21);
signal     fi : std_logic_vector(80 downto 27);
signal     fo : std_logic_vector(81 downto 28);
signal     gi : std_logic_vector(82 downto 37);
signal     go : std_logic_vector(83 downto 38);     
signal     hi : std_logic_vector(88 downto 43);
signal     ho : std_logic_vector(89 downto 44);     


       -- signal definitions 

 begin 
I_singlets: singlets PORT MAP(
		link0 => link0_array,
		link2 => link2_array,
		link3 => link3_array,
		link4 => link4_array,
		link5 => link5_array,
		link6 => link6_array,
		ai => ai,
		ao => ao,
		bi => bi,
		bo => bo,
		ci => ci,
		co => co,
		di => di,
		do => do,
		ei => ei,
		eo => eo,
		fi => fi,
		fo => fo,
		gi => gi,
		go => go,
		hi => hi,
		ho => ho
	);
I_high_tracks: high_tracks port map(
		ai => ai,
		ao => ao,
		bi => bi,
		bo => bo,
		ci => ci,
		co => co,
		di => di,
		do => do,
		ei => ei,
		eo => eo,
		fi => fi,
		fo => fo,
		gi => gi,
		go => go,
		hi => hi,
		ho => ho,
		version => eqs_vers(31 downto 16),
		sector => eqs_vers(15 downto 0),
		m => m);
       -- combinatorial assignments
end track_arch;
