-- doublet_counter.vhd
-- counts the number of set doublets in the home sector

-- jamieson olsen <jamieson@fnal.gov>
-- 11 january 2002; revised 7 August 2002

-- 240 doublets are grouped into six groups of 40 bits each (string_a, string_b, etc)
-- the sum40 module counts the number of bits in each group and outputs a 6-bit unsigned
-- integer.  these 6 integers are concatenated into a 36 bit string (int_string) and 
-- passed on to the pipelined adder (pipe_add).



library ieee;
use ieee.std_logic_1164.all;

entity doublet_counter is
 	port (
		clock:		in 	std_logic;
		load:		in	std_logic;						-- active high
		a:			in	std_logic_vector(32 downto 17);	-- doublets 
		b:			in	std_logic_vector(40 downto 21);
		c:			in	std_logic_vector(48 downto 25);
		d:			in	std_logic_vector(56 downto 29);
		e:			in	std_logic_vector(64 downto 33);
		f:			in	std_logic_vector(72 downto 37);
		g:			in	std_logic_vector(80 downto 41);
		h:			in	std_logic_vector(88 downto 45);
		sum_out:	out std_logic_vector( 2 downto  0)
	);
end doublet_counter;

architecture doublet_counter_arch of doublet_counter is

	-- define registers...

	-- define signals...

		signal int_string:	std_logic_vector(35 downto 0);

		signal string_a:	std_logic_vector(39 downto 0);
		signal string_b:	std_logic_vector(39 downto 0);
		signal string_c:	std_logic_vector(39 downto 0);
		signal string_d:	std_logic_vector(39 downto 0);
		signal string_e:	std_logic_vector(39 downto 0);
		signal string_f:	std_logic_vector(39 downto 0);

	-- declare components...

		component pipe_add
		 	port (
				clock:	in 	std_logic;
				load:	in	std_logic;
				d: 		in 	std_logic_vector(35 downto 0);
				q: 		out std_logic_vector( 2 downto 0)
			);
		end component;

		component sum40
		    port (
		        a:	in  std_logic_vector(39 downto 0);
		        y:	out std_logic_vector( 5 downto 0)
		    );
		end component;
		
begin

	-- combinatorial stuff...
		string_a(39 downto 0)	<= a(32 downto 17) & b(40 downto 21) & c(48 downto 45);
		string_b(39 downto 0)	<= c(44 downto 25) & d(56 downto 37);
		string_c(39 downto 0)	<= d(36 downto 29) & e(64 downto 33);
		string_d(39 downto 0)	<= f(72 downto 37) & g(80 downto 77);
		string_e(39 downto 0)	<= g(76 downto 41) & h(88 downto 85);
		string_f(39 downto 0)	<= h(84 downto 45);

	-- sequential stuff...
	-- instantiate components...

		counter_a: sum40
		    port map(
		        a(39 downto  0)	=> string_a(39 downto 0),
		        y( 5 downto  0)	=> int_string(5 downto 0)
		    );

		counter_b: sum40
		    port map(
		        a(39 downto  0)	=> string_b(39 downto 0),
		        y( 5 downto  0)	=> int_string(11 downto 6)
		    );

		counter_c: sum40
		    port map(
		        a(39 downto  0)	=> string_c(39 downto 0),
		        y( 5 downto  0)	=> int_string(17 downto 12)
		    );

		counter_d: sum40
		    port map(
		        a(39 downto  0)	=> string_d(39 downto 0),
		        y( 5 downto  0)	=> int_string(23 downto 18)
		    );

		counter_e: sum40
		    port map(
		        a(39 downto  0)	=> string_e(39 downto 0),
		        y( 5 downto  0)	=> int_string(29 downto 24)
		    );

		counter_f: sum40
		    port map(
		        a(39 downto  0)	=> string_f(39 downto 0),
		        y( 5 downto  0)	=> int_string(35 downto 30)
		    );

		piplined_adder: pipe_add
		 	port map(
				clock			=> clock,
				load			=> load,
				d(35 downto 0)	=> int_string(35 downto 0),
				q(2 downto 0)	=> sum_out(2 downto 0)
			);

end doublet_counter_arch;
